@@ -53,31 +53,36 @@ class Tokenizer(tokenization_abc.Tokenization):
     that document records. (e.g. title, author, ...).
     """
 
-    def __init__(self, docs: Iterable[dict], domain_stopwords: List[str] = None) -> None:
+    def __init__(self, domain_stopwords: List[str] = None) -> None:
         """
         Creates a new Tokenizer object.
 
         Args:
-            docs (Iterable[dict]): Stream of json documents as dicts. <br/> Each document must have a key where the text of the document is stored.
             domain_stopwords (List[str], optional, default=None): You can provide a list of arbitrary stopwords specific to your corpus domain.
         """
 
         # super __init__
-        super(Tokenizer, self).__init__(docs)
+        super(Tokenizer, self).__init__()
 
         # add some stopwords to nlp list.
         self.nlp.stopwords_list(include=domain_stopwords)
 
-    def ngrams2tokens(self, n: Union[int, Iterable[int]], pos: Tuple[str, ...] = ("PROPN", "VERB", "NOUN", "ADJ"),
+        self.corpus_ngrams = None
+
+    def ngrams2tokens(self, n: Union[int, Iterable[int]], docs_in:Iterable[Dict[str, str]], docs_out: Iterable[Dict[str, str]]=None, pos: Tuple[str, ...] = ("PROPN", "VERB", "NOUN", "ADJ"),
                       corpus_threshold: int = 50,
                       doc_threshold: int = 0,
                       len_gram: int = 3, include: Tuple[str, ...] = None, replace: bool = True) -> Dict[int, str]:
         """
-        Returns frequent n-grams in the corpus. They will be considered as tokens during the Tokenization task.
+        Calculates frequent n-grams in the corpus. They will be considered as unique tokens during the Tokenization task. <br/>
+        Frequent n-grams will be calculated based on the documents in the input stream (docs_in). <br/>
+        Frequent n-grams will be considered as unique tokens in the output stream documents (docs_out).
 
         Args:
-           n (Union[int, Iterable[int]]): If int, the size of n-grams. (e.g. n=2, bigrams) <br/> If Iterable[int], the sizes of n-grams. (e.g. n=[2,3,5], bigrams, trigrams, fivegrams)
-           pos (Tuple[str, ...], optional): default ('PROPN', 'VERB', 'NOUN', 'ADJ'). <br/> Part of speech of the first and the last token that make up n-grams. <br/> Filters the most informative n-grams. <br/> If None, the filter will be ignored.
+            docs_in (Iterable[Dict[str, str]]): Stream of input json documents as dicts. <br/> Each document must have a key where the text of the document is stored. Frequent ngrams will be calculated by scrolling this stream.
+            docs_out (Iterable[Dict[str, str]], optional, default=None): Stream of output json documents as dicts. <br/> Each document must have a key where the text of the document is stored. If not None, frequent ngrams will be considered as unique tokens for each document in the stream. Each document will be yielded as output of this function.
+            n (Union[int, Iterable[int]]): If int, the size of n-grams. (e.g. n=2, bigrams) <br/> If Iterable[int], the sizes of n-grams. (e.g. n=[2,3,5], bigrams, trigrams, fivegrams)
+            pos (Tuple[str, ...], optional): default ('PROPN', 'VERB', 'NOUN', 'ADJ'). <br/> Part of speech of the first and the last token that make up n-grams. <br/> Filters the most informative n-grams. <br/> If None, the filter will be ignored.
 
                    Example:
                    pos = ("PROPN", "VERB", "NOUN", "ADJ")
@@ -131,9 +136,11 @@ class Tokenizer(tokenization_abc.Tokenization):
                       "New York New_York is the most populous city in the United States United_States."
 
         Returns:
-            Dict[str, int]: N-grams as keys, frequencies as values. <br/> The frequencies of the n-grams included arbitrarily are set to 0.
+            Iterable[Dict[str, str]]: Stream of json documents (docs_out) with frequent n-grams considered as unique token.
 
 
+            It will be setted _self.corpus_ngram_ (Dict[str, int]): N-grams as keys, frequencies as values. <br/> The frequencies of the n-grams included arbitrarily are set to 0.
+
                     Example:
                     include = ("Giorgio Napolitano")
 
@@ -152,15 +159,13 @@ class Tokenizer(tokenization_abc.Tokenization):
             sizes = [n]
 
         ngram_frequencies = defaultdict(int)
-        bar = tqdm(total=len(self.docs), desc='N-grams Retrieval in progress...', position=0, leave=True)
+
         # for each corpus doc
-        for doc in self.docs:
+        for doc in tqdm(docs_in, desc='N-gram Retrieval in progress...', position=0, leave=True):
             text = doc[self.text_key]
             for n in sizes:
                 for ngram, frequency in self.nlp.ngrams(text, n, pos=tuple(), threshold=doc_threshold).items():
                     ngram_frequencies[ngram] += frequency
-            bar.update(1)
-        bar.close()
 
         # Select the n grams you're interested in
         ngram_frequencies = {ngram: frequency for ngram, frequency in ngram_frequencies.items() if
@@ -184,24 +189,28 @@ class Tokenizer(tokenization_abc.Tokenization):
                     result[ngram] = 0
 
         # sorted by len: first we replace the longer n-grams, then shorter ones.
-        result = dict(sorted(result.items(), key=lambda x: len(x[0]), reverse=True))
+        self.corpus_ngrams = dict(sorted(result.items(), key=lambda x: len(x[0]), reverse=True))
 
         # updates doc: replaces n-grams with tokens
-        for doc in self.docs:
-            for ngram in result:
+        if docs_out is None:
+            return
+
+        for doc in docs_out:
+            for ngram in self.corpus_ngrams:
                 if replace:
                     doc[self.text_key] = doc[self.text_key].replace(ngram, ngram.replace(' ', '_'))
                 else:
                     doc[self.text_key] = doc[self.text_key].replace(ngram, ngram + ' ' + ngram.replace(' ', '_'))
-        return result
+            yield doc
 
-    def run(self, doc_id: int = 0, token_id: int = 0, sent_id: int = 0, para_id: int = 0,
+    def run(self, docs:Iterable[Dict[str, str]], doc_id: int = 0, token_id: int = 0, sent_id: int = 0, para_id: int = 0,
             para_size: int = 3, index_doc: bool = True, index_sent: bool = True,
             index_para: bool = True) -> Iterable[Dict[str, dict]]:
         """
         Runs the tokenization for each document of the corpus.
 
         Args:
+            docs (Iterable[Dict[str, str]]): Stream of json documents as dicts. <br/> Each document must have a key where the text of the document is stored. Each document will be tokenized.
             doc_id (int, optional, default=0): Unique document identifier. <br/> Numeric id of documents. <br/> A progressive id will be assigned to the documents starting from this number.
             token_id (int, optional, default=0): Unique token identifier. <br/> Numeric id of tokens. <br/> A progressive id will be assigned to the tokens starting from this number.
             sent_id (int, optional, default=0): Unique sentence identifier. <br/> Numeric id of sentences. <br/> A progressive id will be assigned to the sentences starting from this number.
@@ -217,8 +226,8 @@ class Tokenizer(tokenization_abc.Tokenization):
                 Example:
                 [index_sent == True, index_para == False, index_doc == True]
 
-                >>> t = Tokenizer(docs=my_docs, domain_stopwords=my_domain_stopwords)
-                >>> records = t.run():
+                >>> t = Tokenizer(domain_stopwords=my_domain_stopwords)
+                >>> records = t.run(docs=my_docs):
                 >>> next(records).keys()
                 dict_keys(["tokens", "sentences", "documents"])
 
@@ -262,8 +271,7 @@ class Tokenizer(tokenization_abc.Tokenization):
         sent_list, para_list = list(), list()
 
         # tokenization of each doc
-        bar = tqdm(total=len(list(self.docs)), desc='Tokenization in progress...', position=0, leave=True)
-        for doc in self.docs:
+        for doc in tqdm(docs, desc='Tokenization in progress...', position=0, leave=True):
 
             text = doc[self.text_key]
 
@@ -338,6 +346,4 @@ class Tokenizer(tokenization_abc.Tokenization):
             if index_doc:
                 tokenization_['documents'] = doc_records
 
-            bar.update(1)
             yield tokenization_
-        bar.close()
